<?php
/**
 * stl to png conversion script by Tman
 * 
 * Input: 			.zip file with .png and .stl files in it (.png is optional)
 * Output: 			the .stl files and matching .png files will be in the subfolder /good 
 * Requirements:	povray
 * 					stl2pov
 * 					render.pov (should be in the same directory as this file)
 * 					php needs write access in the folder this file is in and its subfolders
 *
 * How to install these (tested on Kubuntu):
 * download povray-3.5 and install it (http://packages.ubuntu.com/dapper/povray-3.5)
 * install gcc (sudo apt-get install gcc)
 * download and extract stl2pov (http://www.xs4all.nl/~rsmith/software/#stl2pov)
 * open MakeFile
 * uncomment CC = gcc
 * use the make command when you are in the correct directory to compile the program
 * it didn't work for me so i changed this:
 *  -instead of the include, i just placed everything in the include in Makefile
 *  -i put the following 2 lines in comment:.if !defined(NOPORTDOCS) and .endif
 * then make did work, so it got compiled
 * when its compiled, use "sudo make install" to install it
 */

// Location of the zip, this will be changed into a parameter later on.
$filename="/home/tman/Documents/test.zip";//change to location of the zip

//Try to open the zip
$zip = new ZipArchive;
$res = $zip->open($filename);
if ($res === TRUE) {
	// extract the files
	$zip->extractTo('unzip/');
	$zip->close();
	// open this directory
	$myDirectory = opendir("unzip");
	// get filenames
	$fileNames=array();
	while($entryName = readdir($myDirectory)) {
		$fileNames[] = $entryName;
	}
	closedir($myDirectory);
	//loop over all the files
	foreach($fileNames as $file){
		//check if it is an stl file
		if(strstr($file, '.stl')){
			unset($picture);
			$picName=str_replace(".stl", ".png", $file);
			//search for the picture to go with it
			foreach($fileNames as $pic){
				if($pic==$picName){
					$picture=$pic;
					print "Matching image found for $file.<br />\n";
				}
			}
			//there was no picture, so it needs to be created
			if(!isset($picture)){
				convert("unzip/$file", "unzip/$picName");
			}
			
			//move the good files to an other directory
			//this has to be changed to the right directory for mediawiki later on
			rename("unzip/$file", "good/$file");
			rename("unzip/$picName", "good/$picName");
			
			//for debugging: print a link to the picture, this will be removed in the final version
			print "<img src='good/$picName' /><br />\n";
		}
	}
	
	//Delete all the remaining files
	foreach($fileNames as $file){
		if(file_exists('unzip/'.$file))
			unlink('unzip/'.$file);
	}
} else {
	print "Unable to open zip file";
}


function convert($filename, $output){
	print "Converting $filename to $output.<br />";
	//Convert the stl to an includable file for povray
	exec("stl2pov -s $filename > render.inc");

	//Rename the model in the newly created .inc file to MODELNAME
	$contents = file_get_contents('render.inc');
	$startReplacing = strpos($contents, '#declare');
	$stopReplacing = strpos($contents, '{', $startReplacing);
	$toReplace=substr($contents, $startReplacing, $stopReplacing-$startReplacing);
	$contents = str_replace($toReplace, '#declare MODELNAME = mesh ', $contents);
	$handle=fopen('render.inc', 'w');
	fwrite($handle, $contents);
	fclose($handle);

	//Render the file
	exec("povray -i\"render.pov\" +W600 +H600 -o\"$output\" +GA -D");

	//Delete the include-file
	unlink('render.inc');
}

?>
